<?php

namespace EventPortal\Tests\Feature\Admin\EventSignIn;

use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\EventDelegate\Actions\AssociateToEvent;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class SignInUpdateTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots, SeedRolesAndPermissions;

    /** @test */
    public function can_toggle_sign_in_state(): void
    {
        $event = factory(Event::class)->create();
        $eventDelegate = factory(EventDelegate::class)->create();
        Pathway::generate($event, Pathway::DEFAULT, 'The default pathway');

        app(AssociateToEvent::class)->execute($event, [$eventDelegate->id]);

        $this->actingAsSuperAdmin()
            ->post(route('admin.event.sign-in.update', [$event, $eventDelegate]));

        $this->assertTrue($eventDelegate->isSignedInAt($event));
    }

    /** @test */
    public function updates_delegates_status_to_attended()
    {
        $event = factory(Event::class)->create();
        $eventDelegate = factory(EventDelegate::class)->state('registered')->create();
        Pathway::generate($event, Pathway::DEFAULT, 'The default pathway');

        app(AssociateToEvent::class)->execute($event, [$eventDelegate->id]);

        $this->actingAsSuperAdmin()
            ->post(route('admin.event.sign-in.update', [$event, $eventDelegate]));

        $this->assertTrue($eventDelegate->isSignedInAt($event));
        $this->assertEquals('attended', $eventDelegate->fresh()->status);
    }
}
